@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
+import logging
 import os
-from typing import List, Union
 import platform
+from typing import List, Union, Type
 
 import pandas as pd
 from sqlalchemy import create_engine
@@ -13,10 +14,13 @@ from sqlalchemy.orm import sessionmaker, Session
 
 from zvt import zvt_env
 from zvt.contract import IntervalLevel, EntityMixin
+from zvt.contract import Mixin
 from zvt.contract import zvt_context
 from zvt.utils.pd_utils import pd_is_not_null, index_df
 from zvt.utils.time_utils import to_pd_timestamp
 
+logger = logging.getLogger(__name__)
+
 
 def get_db_name(data_schema: DeclarativeMeta) -> str:
     """
@@ -244,6 +248,19 @@ def common_filter(query: Query,
     return query
 
 
+def del_data(data_schema: Type[Mixin], filters: List = None, provider=None):
+    if not provider:
+        provider = data_schema.providers[0]
+
+    session = get_db_session(provider=provider, data_schema=data_schema)
+    query = session.query(data_schema)
+    if filters:
+        for f in filters:
+            query = query.filter(f)
+    query.delete()
+    session.commit()
+
+
 def get_data(data_schema,
              ids: List[str] = None,
              entity_ids: List[str] = None,
@@ -389,25 +406,26 @@ def df_to_db(df: pd.DataFrame,
              data_schema: DeclarativeMeta,
              provider: str,
              force_update: bool = False,
-             sub_size: int = 5000) -> object:
+             sub_size: int = 5000,
+             drop_duplicates: bool = False) -> object:
     """
     FIXME:improve
     store the df to db
 
     :param df:
-    :type df:
     :param data_schema:
-    :type data_schema:
     :param provider:
-    :type provider:
     :param force_update:
-    :type force_update:
     :param sub_size:
+    :param drop_duplicates:
     :return:
-    :rtype:
     """
     if not pd_is_not_null(df):
-        return
+        return 0
+
+    if drop_duplicates and df.duplicated(subset='id').any():
+        logger.warning(f'remove duplicated:{df[df.duplicated()]}')
+        df = df.drop_duplicates(subset='id', keep='last')
 
     db_engine = get_db_engine(provider, data_schema=data_schema)
 
@@ -416,7 +434,7 @@ def df_to_db(df: pd.DataFrame,
 
     if not cols:
         print('wrong cols')
-        return
+        return 0
 
     df = df[cols]
 
@@ -432,6 +450,8 @@ def df_to_db(df: pd.DataFrame,
     else:
         step_size = 1
 
+    saved = 0
+
     for step in range(step_size):
         df_current = df.iloc[sub_size * step:sub_size * (step + 1)]
         if force_update:
@@ -451,7 +471,11 @@ def df_to_db(df: pd.DataFrame,
             if pd_is_not_null(current):
                 df_current = df_current[~df_current['id'].isin(current['id'])]
 
-        df_current.to_sql(data_schema.__tablename__, db_engine, index=False, if_exists='append')
+        if pd_is_not_null(df_current):
+            saved = saved + len(df_current)
+            df_current.to_sql(data_schema.__tablename__, db_engine, index=False, if_exists='append')
+
+    return saved
 
 
 def get_entities(
