@@ -2,15 +2,14 @@
 import argparse
 
 import pandas as pd
-
 from jqdatapy.api import get_token, get_bars
-from zvt import init_log, zvt_env
-from zvt.api import get_kdata, AdjustType
-from zvt.api.quote import generate_kdata_id, get_kdata_schema
-from zvt.contract import IntervalLevel
+
+from zvt import init_log, zvt_config
+from zvt.api.quote import generate_kdata_id, get_kdata_schema, get_kdata
+from zvt.contract import IntervalLevel, AdjustType
 from zvt.contract.api import df_to_db
 from zvt.contract.recorder import FixedCycleDataRecorder
-from zvt.domain import Stock, StockKdataCommon, Stock1dHfqKdata
+from zvt.domain import Stock, StockKdataCommon, Stock1dHfqKdata, Stock1wkHfqKdata
 from zvt.recorders.joinquant.common import to_jq_trading_level, to_jq_entity_id
 from zvt.utils.pd_utils import pd_is_not_null
 from zvt.utils.time_utils import to_time_str, now_pd_timestamp, TIME_FORMAT_DAY, TIME_FORMAT_ISO8601
@@ -30,6 +29,7 @@ class JqChinaStockKdataRecorder(FixedCycleDataRecorder):
                  exchanges=['sh', 'sz'],
                  entity_ids=None,
                  codes=None,
+                 day_data=False,
                  batch_size=10,
                  force_update=True,
                  sleeping_time=0,
@@ -49,12 +49,12 @@ class JqChinaStockKdataRecorder(FixedCycleDataRecorder):
         self.data_schema = get_kdata_schema(entity_type='stock', level=level, adjust_type=adjust_type)
         self.jq_trading_level = to_jq_trading_level(level)
 
-        super().__init__('stock', exchanges, entity_ids, codes, batch_size, force_update, sleeping_time,
+        super().__init__('stock', exchanges, entity_ids, codes, day_data, batch_size, force_update, sleeping_time,
                          default_size, real_time, fix_duplicate_way, start_timestamp, end_timestamp, close_hour,
                          close_minute, level, kdata_use_begin_time, one_day_trading_minutes)
         self.adjust_type = adjust_type
 
-        get_token(zvt_env['jq_username'], zvt_env['jq_password'], force=True)
+        get_token(zvt_config['jq_username'], zvt_config['jq_password'], force=True)
 
     def generate_domain_id(self, entity, original_data):
         return generate_kdata_id(entity_id=entity.id, timestamp=original_data['timestamp'], level=self.level)
@@ -131,26 +131,15 @@ class JqChinaStockKdataRecorder(FixedCycleDataRecorder):
 
             df['id'] = df[['entity_id', 'timestamp']].apply(generate_kdata_id, axis=1)
 
+            df = df.drop_duplicates(subset='id', keep='last')
+
             df_to_db(df=df, data_schema=self.data_schema, provider=self.provider, force_update=self.force_update)
 
         return None
 
 
-__all__ = ['JqChinaStockKdataRecorder']
-
 if __name__ == '__main__':
-    parser = argparse.ArgumentParser()
-    parser.add_argument('--level', help='trading level', default='1d', choices=[item.value for item in IntervalLevel])
-    parser.add_argument('--codes', help='codes', default=['000001'], nargs='+')
+    Stock1wkHfqKdata.record_data(codes=['300999'])
 
-    args = parser.parse_args()
-
-    level = IntervalLevel(args.level)
-    codes = args.codes
-
-    init_log('jq_china_stock_{}_kdata.log'.format(args.level))
-    JqChinaStockKdataRecorder(level=level, sleeping_time=0, codes=codes, real_time=False,
-                              adjust_type=AdjustType.hfq).run()
-
-    print(get_kdata(entity_id='stock_sz_000001', limit=10, order=Stock1dHfqKdata.timestamp.desc(),
-                    adjust_type=AdjustType.hfq))
+# the __all__ is generated
+__all__ = ['JqChinaStockKdataRecorder']
